from __future__ import absolute_import, unicode_literals

from django.contrib import admin
from ordered_model.admin import OrderedModelAdmin

from .models import AppGroup, AppLink


class AppLinkInline(admin.TabularInline):
    model = AppLink
    fields = ('name', 'link', )
    fk_name = 'app_group'
    extra = 0


@admin.register(AppGroup)
class AppGroupAdmin(OrderedModelAdmin):
    list_display = ('name', 'move_up_down_links', )
    fields = ('name', 'slug', 'models', )
    prepopulated_fields = {'slug': ('name', )}
    filter_horizontal = ('models', )
    inlines = (AppLinkInline, )



from __future__ import absolute_import, unicode_literals

import django
from django.contrib.admin import site
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _
from ordered_model.models import OrderedModel

from . import settings


class AppGroupQuerySet(models.QuerySet):
    def get_by_natural_key(self, slug):
        return self.get(slug=slug)

    def as_list(self, request, include_remaining=True):
        # Convert to convienent dict
        model_dicts = {}

        if django.VERSION < (1, 9):
            from .compat.django18 import get_app_list
            original_app_list = get_app_list(site, request)
        else:
            original_app_list = site.get_app_list(request)

        for app in original_app_list:
            for model in app['models']:
                key = '{}.{}'.format(app['app_label'], model['object_name'].lower())  # noqa
                model_dict = model.copy()
                model_dict.update({
                    'app_label': app['app_label'],
                    'app_url': app['app_url'],
                    'has_module_perms': app['has_module_perms'],
                })
                model_dicts[key] = model_dict

        added = []

        # Create new list based on our groups, using the model_dicts constructed above.  # noqa
        result = []
        app_list = self.prefetch_related('models', 'applink_set')
        for app in app_list:
            models = []
            for model in app.models.all():
                key = '{}.{}'.format(model.app_label, model.model)
                o = model_dicts.get(key)
                if o:
                    models.append(o)
                    added.append(key)

            for app_link in app.applink_set.all():
                models.append({
                    'name': app_link.name,
                    'app_label': app.slug,
                    'admin_url': app_link.link,
                })

            if models:
                result.append({
                    'name': app.name,
                    'app_label': app.slug,
                    'models': sorted(models, key=lambda m: m['name'])
                })

        if settings.AUTO_CREATE_APP_GROUP:
            other = set(model_dicts.keys()).difference(set(added))
            for model in other:
                names = model.split('.')
                app_group, created = AppGroup.objects.get_or_create(
                    slug=names[0], name=names[0].title())
                contenttype = ContentTypeProxy.objects.get(
                    app_label=names[0], model=names[1])
                app_group.models.add(contenttype)
        elif include_remaining:
            other = set(model_dicts.keys()).difference(set(added))
            if other:
                result.append({
                    'name': _('Miscellaneous'),
                    'app_label': 'misc',
                    'models': sorted(
                        [model_dict for key, model_dict in model_dicts.items() if key in other],  # noqa
                        key=lambda m: m['name'],
                    )
                })

        return result


class AppLinkQuerySet(models.QuerySet):
    def get_by_natural_key(self, app_group, link):
        return self.get(app_group=app_group, link=link)


@python_2_unicode_compatible
class ContentTypeProxy(ContentType):
    class Meta:
        proxy = True
        ordering = ('app_label', 'model')

    def __str__(self):
        return '{}.{}'.format(self.app_label, capfirst(self.model))


@python_2_unicode_compatible
class AppGroup(OrderedModel):
    name = models.CharField(_('name'), max_length=200)
    slug = models.SlugField(_('slug'), unique=True)
    models = models.ManyToManyField(ContentTypeProxy, blank=True)

    objects = AppGroupQuerySet.as_manager()

    class Meta(OrderedModel.Meta):
        verbose_name = _('application group')
        verbose_name_plural = _('application groups')

    def natural_key(self):
        return (self.slug, )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class AppLink(OrderedModel):
    app_group = models.ForeignKey(AppGroup, on_delete=models.CASCADE)
    name = models.CharField(max_length=200)
    link = models.CharField(max_length=200)

    objects = AppLinkQuerySet.as_manager()

    class Meta(OrderedModel.Meta):
        verbose_name = _('application link')
        verbose_name_plural = _('application links')
        unique_together = (
            ('app_group', 'link'),
        )

    def natural_key(self):
        return (self.app_group, self.link)

    def __str__(self):
        return self.name



# encoding: utf-8

from __future__ import absolute_import, unicode_literals

from django.test import TestCase, override_settings

from django_admin_index.apps import check_admin_index_app, check_admin_index_context_processor
from django_admin_index.models import AppGroup, AppLink, ContentTypeProxy


class AdminIndexTests(TestCase):
    def setUp(self):
        # Create new group...
        self.app_group = AppGroup.objects.create(name='My group', slug='my-group')
        # ...find the content type for model User (it needs to be registered in the admin)
        self.ct_user = ContentTypeProxy.objects.get(app_label='auth', model='user')
        # ...and this content type to the new group.
        self.app_group.models.add(self.ct_user)
        # ...and add a link to the same group.
        self.app_link = AppLink.objects.create(
            name='Support', link='https://www.maykinmedia.nl', app_group=self.app_group)

    def test_app_group_str(self):
        self.assertEqual(str(self.app_group), 'My group')

    def test_app_link_str(self):
        self.assertEqual(str(self.app_link), 'Support')

    def test_app_content_type_proxy_str(self):
        self.assertEqual(str(self.ct_user), 'auth.User')

    def test_check_admin_index_app_success(self):
        result = check_admin_index_app([])
        self.assertEqual(len(result), 0)

    @override_settings(INSTALLED_APPS=['django.contrib.admin', 'django_admin_index'])
    def test_check_admin_index_app_incorrect_order(self):
        result = check_admin_index_app([])
        self.assertEqual(len(result), 1)

    @override_settings(INSTALLED_APPS=['django_admin_index'])
    def test_check_admin_index_app_missing(self):
        result = check_admin_index_app([])
        self.assertEqual(len(result), 1)

    def test_check_admin_index_context_process_success(self):
        result = check_admin_index_context_processor([])
        self.assertEqual(len(result), 0)

    @override_settings(TEMPLATES=[{'OPTIONS': {'context_processors': []}}])
    def test_check_admin_index_context_process_missing(self):
        result = check_admin_index_context_processor([])
        self.assertEqual(len(result), 1)



from __future__ import absolute_import, unicode_literals

import django

from . import settings
from .models import AppGroup

if django.VERSION >= (1, 11):
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse


def dashboard(request):
    if hasattr(request, 'user') and hasattr(request, 'path') and request.path == reverse('admin:index'):  # noqa
        show_remaining_apps = settings.SHOW_REMAINING_APPS or \
            (settings.SHOW_REMAINING_APPS_TO_SUPERUSERS and request.user.is_superuser)  # noqa

        app_list = AppGroup.objects.as_list(request, show_remaining_apps)

        return {
            'dashboard_app_list': app_list,
        }

    return {}



# encoding: utf-8

from __future__ import absolute_import, unicode_literals

import django
from django.contrib.auth.models import AnonymousUser, Permission, User
from django.test import RequestFactory, TestCase

from django_admin_index.context_processors import dashboard
from django_admin_index.models import AppGroup, AppLink

if django.VERSION >= (1, 11):
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse


class AdminIndexAppLinkTests(TestCase):
    def setUp(self):
        # Create new group...
        self.app_group = AppGroup.objects.create(name='My group', slug='my-group')
        # ...and add a link to the same group.
        self.app_link = AppLink.objects.create(
            name='Support', link='https://www.maykinmedia.nl', app_group=self.app_group)

        self.factory = RequestFactory()

        self.superuser = self._create_user(username='superuser', is_staff=True, is_superuser=True)

    def _create_user(self, **kwargs):
        options = {
            'username': 'maykin',
            'email': 'info@maykinmedia.nl',
            'password': 'top_secret',
            'is_staff': False,
            'is_superuser': False,
        }
        options.update(kwargs)

        return User.objects._create_user(**options)

    def test_as_list_structure(self):
        request = self.factory.get(reverse('admin:index'))
        request.user = self.superuser

        result = AppGroup.objects.as_list(request, False)
        self.assertEqual(len(result), 1)

        app = result[0]
        app_model = app['models'][0]

        self.assertEqual(app_model, {
            'name': self.app_link.name,
            'app_label': self.app_group.slug,
            'admin_url': self.app_link.link,
        })

    def test_as_list_without_include_remaining(self):
        request = self.factory.get(reverse('admin:index'))
        request.user = self.superuser

        result = AppGroup.objects.as_list(request, False)
        self.assertEqual(len(result), 1)

        app = result[0]

        self.assertEqual(app['app_label'], self.app_group.slug)
        self.assertEqual(len(app['models']), 1)

    def test_as_list_with_include_remaining(self):
        request = self.factory.get(reverse('admin:index'))
        request.user = self.superuser

        result = AppGroup.objects.as_list(request, True)
        self.assertEqual(len(result), 2)

        self.assertSetEqual(
            set([a['app_label'] for a in result]),
            {self.app_group.slug, 'misc'}
        )

        app_my_group = [a for a in result if a['app_label'] == self.app_group.slug][0]
        self.assertEqual(len(app_my_group['models']), 1)
        self.assertSetEqual(
            set(m['name'] for m in app_my_group['models']),
            {'Support', }
        )

        app_misc = [a for a in result if a['app_label'] == 'misc'][0]
        self.assertEqual(len(app_misc['models']), 3)
        self.assertSetEqual(
            set(m['object_name'] for m in app_misc['models']),
            {'User', 'Group', 'AppGroup', }
        )

    def test_context_anonymous(self):
        request = self.factory.get(reverse('admin:index'))
        request.user = AnonymousUser()

        context = dashboard(request)
        self.assertIn('dashboard_app_list', context)
        # The AppLink is shown to everyone. There are no permissions set.
        self.assertEqual(len(context['dashboard_app_list']), 1)

    def test_context_user(self):
        request = self.factory.get(reverse('admin:index'))
        request.user = self._create_user()

        context = dashboard(request)
        self.assertIn('dashboard_app_list', context)
        # The AppLink is shown to everyone. There are no permissions set.
        self.assertEqual(len(context['dashboard_app_list']), 1)

    def test_context_staff_user(self):
        request = self.factory.get(reverse('admin:index'))
        user = self._create_user(is_staff=True)
        user.user_permissions.add(*Permission.objects.all())
        request.user = user

        context = dashboard(request)
        self.assertIn('dashboard_app_list', context)
        self.assertEqual(len(context['dashboard_app_list']), 1)

    # @override_settings(ADMIN_INDEX_SHOW_REMAINING_APPS=True)
    # def test_context_staffuser_with_show_true(self):
    #     request = self.factory.get(reverse('admin:index'))
    #     user = self._create_user(is_staff=True)
    #     user.user_permissions.add(*Permission.objects.all())
    #     request.user = user
    #
    #     context = dashboard(request)
    #     self.assertIn('dashboard_app_list', context)
    #     self.assertEqual(len(context['dashboard_app_list']), 2)

    def test_context_superuser(self):
        request = self.factory.get(reverse('admin:index'))
        request.user = self.superuser

        context = dashboard(request)
        self.assertIn('dashboard_app_list', context)
        self.assertEqual(len(context['dashboard_app_list']), 2)

    # @override_settings(ADMIN_INDEX_SHOW_REMAINING_APPS_TO_SUPERUSERS=False)
    # def test_context_superuser_with_show_false(self):
    #     request = self.factory.get(reverse('admin:index'))
    #     request.user = self.superuser
    #
    #     context = dashboard(request)
    #     self.assertIn('dashboard_app_list', context)
    #     self.assertEqual(len(context['dashboard_app_list']), 1)



# encoding: utf-8

from __future__ import absolute_import, unicode_literals

from unittest import skipIf

import django
from django.contrib.auth.models import User
from django.test import TestCase

if django.VERSION >= (1, 11):
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse


class AdminIndexIntegrationTests(TestCase):
    def setUp(self):
        self.superuser = User.objects._create_user(
            username='superuser', email='user@example.com', password='top_secret', is_staff=True, is_superuser=True)
        self.assertTrue(self.client.login(username=self.superuser.username, password='top_secret'))

        self.auth_app_list_url = reverse('admin:app_list', kwargs={'app_label': 'auth'})

    def test_app_groups_in_context(self):
        response = self.client.get(reverse('admin:index'))

        self.assertIn('dashboard_app_list', response.context)
        self.assertGreater(len(response.context['dashboard_app_list']), 0)

    def test_no_app_groups_in_context_outside_index(self):
        response = self.client.get(reverse('admin:auth_user_changelist'))

        self.assertNotIn('dashboard_app_list', response.context)

    @skipIf(django.VERSION < (1, 9), 'Django < 1.9 does not support template origins.')
    def test_app_groups_in_index(self):
        response = self.client.get(reverse('admin:index'))

        template_path = response.templates[0].origin.name.replace('\\', '/')
        self.assertTrue(template_path.endswith('django_admin_index/templates/admin/index.html'), template_path)

    def test_no_app_list_links_index(self):
        response = self.client.get(reverse('admin:index'))
        html = response.content.decode('utf-8')

        self.assertNotIn('{}"'.format(self.auth_app_list_url), html)

    def test_no_app_list_links_change_form(self):
        response = self.client.get(reverse('admin:auth_user_change', args=(self.superuser.pk, )))
        html = response.content.decode('utf-8')

        self.assertNotIn('{}"'.format(self.auth_app_list_url), html)

    def test_no_app_list_links_change_list(self):
        response = self.client.get(reverse('admin:auth_user_changelist'))
        html = response.content.decode('utf-8')

        self.assertNotIn('{}"'.format(self.auth_app_list_url), html)

    def test_no_app_list_links_delete_confirmation(self):
        response = self.client.get(reverse('admin:auth_user_delete', args=(self.superuser.pk, )))
        html = response.content.decode('utf-8')

        self.assertNotIn('{}"'.format(self.auth_app_list_url), html)

    def test_no_app_list_links_delete_selected_confirmation(self):
        response = self.client.post(reverse('admin:auth_user_changelist'), data={
            'action': 'delete_selected',
            'select_across': 0,
            'index': 0,
            '_selected_action': self.superuser.pk,
        })
        html = response.content.decode('utf-8')

        self.assertNotIn('{}"'.format(self.auth_app_list_url), html)

    def test_no_app_list_links_object_history(self):
        response = self.client.get(reverse('admin:auth_user_history', args=(self.superuser.pk, )))
        html = response.content.decode('utf-8')

        self.assertNotIn('{}"'.format(self.auth_app_list_url), html)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-07-02 08:40
from __future__ import absolute_import, unicode_literals

import django.contrib.contenttypes.models
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='AppGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveIntegerField(db_index=True, editable=False)),
                ('name', models.CharField(max_length=200, verbose_name='name')),
                ('slug', models.SlugField(unique=True, verbose_name='slug')),
            ],
            options={
                'verbose_name_plural': 'application groups',
                'verbose_name': 'application group',
                'abstract': False,
                'ordering': ('order',),
            },
        ),
        migrations.CreateModel(
            name='AppLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveIntegerField(db_index=True, editable=False)),
                ('name', models.CharField(max_length=200)),
                ('link', models.CharField(max_length=200)),
                ('app_group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin_index.AppGroup')),
            ],
            options={
                'verbose_name_plural': 'application links',
                'verbose_name': 'application link',
                'abstract': False,
                'ordering': ('order',),
            },
        ),
        migrations.CreateModel(
            name='ContentTypeProxy',
            fields=[
            ],
            options={
                'proxy': True,
                'ordering': ('app_label', 'model'),
            },
            bases=('contenttypes.contenttype',),
            managers=[
                ('objects', django.contrib.contenttypes.models.ContentTypeManager()),
            ],
        ),
        migrations.AddField(
            model_name='appgroup',
            name='models',
            field=models.ManyToManyField(blank=True, to='admin_index.ContentTypeProxy'),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django.conf import settings as django_settings

SHOW_REMAINING_APPS = getattr(django_settings, 'ADMIN_INDEX_SHOW_REMAINING_APPS', False)

SHOW_REMAINING_APPS_TO_SUPERUSERS = getattr(django_settings, 'ADMIN_INDEX_SHOW_REMAINING_APPS_TO_SUPERUSERS', True)

AUTO_CREATE_APP_GROUP = getattr(django_settings, 'ADMIN_INDEX_AUTO_CREATE_APP_GROUP', False)



from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig
from django.core.checks import Tags, Warning, register
from django.utils.translation import ugettext_lazy as _

__all__ = ['AdminIndexConfig']


class AdminIndexConfig(AppConfig):
    """Default configuration for the django_admin_index app."""

    name = 'django_admin_index'
    label = 'admin_index'
    verbose_name = _('Admin Index')

    def ready(self):
        register(check_admin_index_app, Tags.compatibility)
        register(check_admin_index_context_processor, Tags.compatibility)


def check_admin_index_app(app_configs, **kwargs):
    from django.conf import settings

    issues = []

    try:
        if settings.INSTALLED_APPS.index(AdminIndexConfig.name) > \
                settings.INSTALLED_APPS.index('django.contrib.admin'):
            issues.append(
                Warning('You should put \'{}\' before \'django.contrib.admin\' in your INSTALLED_APPS.'.format(
                    AdminIndexConfig.name
                ))
            )
    except ValueError:
        issues.append(
            Warning('You are missing \'django.contrib.admin\' in your INSTALLED_APPS.')
        )

    return issues


def check_admin_index_context_processor(app_configs, **kwargs):
    from django.conf import settings

    issues = []
    found = False
    context_procesor = '{}.context_processors.dashboard'.format(AdminIndexConfig.name)

    for engine in settings.TEMPLATES:
        if 'OPTIONS' in engine and 'context_processors' in engine['OPTIONS']:
            if context_procesor in engine['OPTIONS']['context_processors']:
                found = True
                break

    if not found:
        issues.append(
            Warning('You are missing \'{}\' in your TEMPLATES.OPTIONS.context_processors.'.format(context_procesor))
        )

    return issues



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.apps import apps
from django.utils.text import capfirst

try:
    from django.urls import NoReverseMatch, reverse
except ImportError:
    from django.core.urlresolvers import NoReverseMatch, reverse


def _build_app_dict(site, request, label=None):
    """
    Builds the app dictionary. Takes an optional label parameters to filter
    models of a specific app.
    """
    app_dict = {}

    if label:
        models = {
            m: m_a for m, m_a in site._registry.items()
            if m._meta.app_label == label
        }
    else:
        models = site._registry

    for model, model_admin in models.items():
        app_label = model._meta.app_label

        has_module_perms = model_admin.has_module_permission(request)
        if not has_module_perms:
            continue

        perms = model_admin.get_model_perms(request)

        # Check whether user has any perm for this module.
        # If so, add the module to the model_list.
        if True not in perms.values():
            continue

        info = (app_label, model._meta.model_name)
        model_dict = {
            'name': capfirst(model._meta.verbose_name_plural),
            'object_name': model._meta.object_name,
            'perms': perms,
        }
        if perms.get('change'):
            try:
                model_dict['admin_url'] = reverse('admin:%s_%s_changelist' % info, current_app=site.name)
            except NoReverseMatch:
                pass
        if perms.get('add'):
            try:
                model_dict['add_url'] = reverse('admin:%s_%s_add' % info, current_app=site.name)
            except NoReverseMatch:
                pass

        if app_label in app_dict:
            app_dict[app_label]['models'].append(model_dict)
        else:
            app_dict[app_label] = {
                'name': apps.get_app_config(app_label).verbose_name,
                'app_label': app_label,
                'app_url': reverse(
                    'admin:app_list',
                    kwargs={'app_label': app_label},
                    current_app=site.name,
                ),
                'has_module_perms': has_module_perms,
                'models': [model_dict],
            }

    if label:
        return app_dict.get(label)
    return app_dict


def get_app_list(site, request):
    """
    Returns a sorted list of all the installed apps that have been
    registered in this site.
    """
    app_dict = _build_app_dict(site, request)

    # Sort the apps alphabetically.
    app_list = sorted(app_dict.values(), key=lambda x: x['name'].lower())

    # Sort the models alphabetically within each app.
    for app in app_list:
        app['models'].sort(key=lambda x: x['name'])

    return app_list



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('admin_index', '0001_initial'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='applink',
            unique_together=set([('app_group', 'link')]),
        ),
    ]
